@@ -25,6 +25,7 @@
 #include "strdup/strdup.h"
 #include "substr/substr.h"
 #include "tempdir/tempdir.h"
+#include "rimraf/rimraf.h"
 #include <curl/curl.h>
 #include <libgen.h>
 #include <limits.h>
@@ -746,7 +747,7 @@ clib_package_new_from_slug_with_package_name(const char *slug, int verbose,
       _debug("failed to cache JSON for: %s/%s@%s", pkg->author, pkg->name,
              pkg->version);
     } else {
-      _debug("cached: %s/%s@%s", pkg->author, pkg->name, pkg->version);
+      _debug("cached json: %s/%s@%s", pkg->author, pkg->name, pkg->version);
     }
   }
 #ifdef HAVE_PTHREADS
@@ -935,7 +936,18 @@ static int fetch_package_file_work(clib_package_t *pkg, const char *dir,
 
   _debug("file URL: %s", url);
 
-  if (!(path = path_join(dir, basename(file)))) {
+  char *base_path = strdup(basename(file));
+
+  if (!base_path) {
+    rc = 1;
+    goto cleanup;
+  }
+
+  path = path_join(dir, base_path);
+
+  free(base_path);
+
+  if (!path) {
     rc = 1;
     goto cleanup;
   }
@@ -1544,13 +1556,6 @@ download:
   }
 #endif
 
-#ifdef HAVE_PTHREADS
-  pthread_mutex_lock(&lock.mutex);
-#endif
-  clib_cache_save_package(pkg->author, pkg->name, pkg->version, pkg_dir);
-#ifdef HAVE_PTHREADS
-  pthread_mutex_unlock(&lock.mutex);
-#endif
 
 install:
   if (pkg->configure) {
@@ -1571,9 +1576,27 @@ install:
     rc = clib_package_install_dependencies(pkg, dir, verbose);
   }
 
+#ifdef HAVE_PTHREADS
+  pthread_mutex_lock(&lock.mutex);
+#endif
+  if (0 == rc) {
+    clib_cache_save_package(pkg->author, pkg->name, pkg->version, pkg_dir);
+    _debug("cached package: %s/%s@%s", pkg->author, pkg->name, pkg->version);
+  }
+#ifdef HAVE_PTHREADS
+  pthread_mutex_unlock(&lock.mutex);
+#endif
+
+
 cleanup:
-  if (pkg_dir)
+  if (pkg_dir) {
+    if (0 != rc) {
+      rimraf(pkg_dir);
+      _debug("deleted inconsistent package dir: %s", pkg_dir);
+    }
+
     free(pkg_dir);
+  }
   if (package_json)
     free(package_json);
   if (iterator)
@@ -1590,6 +1613,18 @@ cleanup:
   }
   fetchs = NULL;
 #endif
+
+#ifdef HAVE_PTHREADS
+  pthread_mutex_lock(&lock.mutex);
+#endif
+  if (0 != rc && pkg) {
+    clib_cache_delete_json(pkg->author, pkg->name, pkg->version);
+    _debug("deleted json cache: %s/%s@%s", pkg->author, pkg->name, pkg->version);
+  }
+#ifdef HAVE_PTHREADS
+  pthread_mutex_unlock(&lock.mutex);
+#endif
+
   return rc;
 }
 
